<?php

namespace EventPortal\Domain\Util\Notification;

use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Notification as LaravelNotification;

class Notification
{
    protected $key;

    /** @var callable|string */
    public $subject;

    /** @var array */
    public $data;

    /** @var \Illuminate\Support\Collection|array|mixed */
    public $recipients;

    /** @var \Illuminate\Support\Collection */
    protected $callables;

    public function __construct(string $key)
    {
        $this->key = $key;
        $this->callables = new Collection();
    }

    /**
     * Set the subject, or register a callback to
     * set it later, once data is available.
     *
     * @param string|callable $subject
     */
    public function subject($subject): self
    {
        if (is_callable($subject)) {
            $this->callables->push('subject');
        }

        $this->subject = $subject;

        return $this;
    }

    /**
     * @param mixed $data
     */
    public function data($data): self
    {
        $this->data = $data;

        return $this;
    }

    /**
     * @param mixed $recipients
     */
    public function to($recipients): self
    {
        $this->recipients = $recipients;

        return $this;
    }

    public function send(): void
    {
        $this->resolveCallbacks();

        LaravelNotification::send($this->recipients, new SystemNotification($this->key, $this->subject, $this->data));
    }

    public function toArray(): array
    {
        return [
            'subject' => $this->subject,
            'data' => $this->data,
            'recipients' => $this->recipients,
        ];
    }

    private function resolveCallbacks(): void
    {
        $this->callables->each(function (string $property): void {
            $this->$property = call_user_func($this->$property, $this->data);
        });
    }
}
