<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\User\Models\User;
use EventPortal\Domain\Util\AdminRender;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\DelegateRender;
use EventPortal\Domain\Util\Menu\Facades\Menu;
use EventPortal\Seeds\RoleSeeder;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Facades\Route;
use Spatie\Snapshots\MatchesSnapshots;

class DelegateRenderTest extends TestCase
{
    use MatchesSnapshots, RefreshDatabase;

    public function setUp(): void
    {
        parent::setUp();
        $this->withoutMiddleware();

        $this->pageHeader = [
            "title" => "Dashboard",
            "description" => "A home page",
            "howToText" => 'Text that isn\'t null',
            "howToVideo" => null,
        ];

        config(['page-headers.admin.dashboard.index' => $this->pageHeader]);

        (new RoleSeeder())->run();
        $this->actingAsAdmin();
    }

    /** @test */
    public function can_include_menu(): void
    {
        $user = factory(User::class)->state('snapshot')->create();
        $this->actingAs($user);

        $event = factory(Event::class)->create(['name' => 'Test Event']);

        $menu = Menu::make('primary')
            ->model($event)
            ->render();

        $viewdata = DelegateRender::component('Some/Page')
            ->key('some-page.index')
            ->withMenu($menu)
            ->getData();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }
}
