<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Util\Menu\Facades\Menu;
use EventPortal\Domain\Util\Menu\Link;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class MenuTest extends TestCase
{
    use MatchesSnapshots, RefreshDatabase;

    /** @test */
    public function can_be_generated()
    {
        $event = factory(Event::class)->create(['name' => 'Test Event']);

        $menu = Menu::make('primary')
            ->model($event)
            ->render();

        $this->assertMatchesJsonSnapshot($menu);
    }

    /** @test */
    public function can_add_links()
    {
        $event = factory(Event::class)->create(['name' => 'Test Event']);

        $menu = Menu::make('primary')
            ->model($event)
            ->addLinks(function () {
                return [
                    Link::make('Something else'),
                ];
            })
            ->render();

        $this->assertMatchesJsonSnapshot($menu);
    }

    /** @test */
    public function can_hide_link()
    {
        $event = factory(Event::class)->create(['name' => 'Test Event']);

        $menu = Menu::make('primary')
            ->model($event)
            ->addLinks(function () {
                return [
                    Link::make('Something else')->canSee(false),
                ];
            })
            ->render();

        $this->assertMatchesJsonSnapshot($menu);
    }

    /** @test */
    public function can_hide_link_callable()
    {
        $event = factory(Event::class)->create(['name' => 'Test Event']);

        $menu = Menu::make('primary')
            ->model($event)
            ->addLinks(function () {
                return [
                    Link::make('Something else')
                        ->canSee(function () {
                            return false;
                        }),
                ];
            })
            ->render();

        $this->assertMatchesJsonSnapshot($menu);
    }
}
